@@ -1,14 +1,16 @@
 """LaTeX builder."""
 
+from __future__ import annotations
+
 import os
 import warnings
 from os import path
-from typing import Any, Dict, Iterable, List, Tuple, Union
+from typing import Any, Iterable
 
 from docutils.frontend import OptionParser
 from docutils.nodes import Node
 
-import sphinx.builders.latex.nodes  # NOQA  # Workaround: import this before writer to avoid ImportError
+import sphinx.builders.latex.nodes  # noqa: F401,E501  # Workaround: import this before writer to avoid ImportError
 from sphinx import addnodes, highlighting, package_dir
 from sphinx.application import Sphinx
 from sphinx.builders import Builder
@@ -19,8 +21,9 @@ from sphinx.config import ENUM, Config
 from sphinx.environment.adapters.asset import ImageAdapter
 from sphinx.errors import NoUri, SphinxError
 from sphinx.locale import _, __
-from sphinx.util import logging, progress_message, status_iterator, texescape
+from sphinx.util import logging, texescape
 from sphinx.util.console import bold, darkgreen  # type: ignore
+from sphinx.util.display import progress_message, status_iterator
 from sphinx.util.docutils import SphinxFileOutput, new_document
 from sphinx.util.fileutil import copy_asset_file
 from sphinx.util.i18n import format_date
@@ -114,9 +117,9 @@ class LaTeXBuilder(Builder):
 
     def init(self) -> None:
         self.babel: ExtBabel = None
-        self.context: Dict[str, Any] = {}
+        self.context: dict[str, Any] = {}
         self.docnames: Iterable[str] = {}
-        self.document_data: List[Tuple[str, str, str, str, str, bool]] = []
+        self.document_data: list[tuple[str, str, str, str, str, bool]] = []
         self.themes = ThemeFactory(self.app)
         texescape.init()
 
@@ -124,16 +127,16 @@ class LaTeXBuilder(Builder):
         self.init_babel()
         self.init_multilingual()
 
-    def get_outdated_docs(self) -> Union[str, List[str]]:
+    def get_outdated_docs(self) -> str | list[str]:
         return 'all documents'  # for now
 
-    def get_target_uri(self, docname: str, typ: str = None) -> str:
+    def get_target_uri(self, docname: str, typ: str | None = None) -> str:
         if docname not in self.docnames:
             raise NoUri(docname, typ)
         else:
             return '%' + docname
 
-    def get_relative_uri(self, from_: str, to: str, typ: str = None) -> str:
+    def get_relative_uri(self, from_: str, to: str, typ: str | None = None) -> str:
         # ignore source path
         return self.get_target_uri(to, typ)
 
@@ -144,7 +147,7 @@ class LaTeXBuilder(Builder):
                               'will be written'))
             return
         # assign subdirs to titles
-        self.titles: List[Tuple[str, str]] = []
+        self.titles: list[tuple[str, str]] = []
         for entry in preliminary_document_data:
             docname = entry[0]
             if docname not in self.env.all_docs:
@@ -170,6 +173,9 @@ class LaTeXBuilder(Builder):
         self.context.update(self.config.latex_elements)
         self.context['release'] = self.config.release
         self.context['use_xindy'] = self.config.latex_use_xindy
+        self.context['booktabs'] = 'booktabs' in self.config.latex_table_style
+        self.context['borderless'] = 'borderless' in self.config.latex_table_style
+        self.context['colorrows'] = 'colorrows' in self.config.latex_table_style
 
         if self.config.today:
             self.context['date'] = self.config.today
@@ -233,11 +239,11 @@ class LaTeXBuilder(Builder):
             self.context['classoptions'] += ',' + self.babel.get_language()
             options = self.babel.get_mainlanguage_options()
             if options:
-                language = r'\setmainlanguage[%s]{%s}' % (options, self.babel.get_language())
+                language = fr'\setmainlanguage[{options}]{{{self.babel.get_language()}}}'
             else:
                 language = r'\setmainlanguage{%s}' % self.babel.get_language()
 
-            self.context['multilingual'] = '%s\n%s' % (self.context['polyglossia'], language)
+            self.context['multilingual'] = f'{self.context["polyglossia"]}\n{language}'
 
     def write_stylesheet(self) -> None:
         highlighter = highlighting.PygmentsBridge('latex', self.config.pygments_style)
@@ -318,7 +324,9 @@ class LaTeXBuilder(Builder):
         self.context['pointsize'] = theme.pointsize
         self.context['wrapperclass'] = theme.wrapperclass
 
-    def assemble_doctree(self, indexfile: str, toctree_only: bool, appendices: List[str]) -> nodes.document:  # NOQA
+    def assemble_doctree(
+        self, indexfile: str, toctree_only: bool, appendices: list[str]
+    ) -> nodes.document:
         self.docnames = set([indexfile] + appendices)
         logger.info(darkgreen(indexfile) + " ", nonl=True)
         tree = self.env.get_doctree(indexfile)
@@ -349,7 +357,7 @@ class LaTeXBuilder(Builder):
         for pendingnode in largetree.findall(addnodes.pending_xref):
             docname = pendingnode['refdocname']
             sectname = pendingnode['refsectname']
-            newnodes: List[Node] = [nodes.emphasis(sectname, sectname)]
+            newnodes: list[Node] = [nodes.emphasis(sectname, sectname)]
             for subdir, title in self.titles:
                 if docname.startswith(subdir):
                     newnodes.append(nodes.Text(_(' (in ')))
@@ -405,12 +413,15 @@ class LaTeXBuilder(Builder):
             copy_asset_file(path.join(self.confdir, filename), self.outdir)
 
     def copy_image_files(self) -> None:
-        if self.images:
+        if self.env.images:
+            converted_images = {*self.env.original_image_uri.values()}
             stringify_func = ImageAdapter(self.app.env).get_original_image_uri
-            for src in status_iterator(self.images, __('copying images... '), "brown",
-                                       len(self.images), self.app.verbosity,
+            for src in status_iterator(self.env.images, __('copying images... '), "brown",
+                                       len(self.env.images), self.app.verbosity,
                                        stringify_func=stringify_func):
-                dest = self.images[src]
+                if src in converted_images:
+                    continue
+                _docnames, dest = self.env.images[src]
                 try:
                     copy_asset_file(path.join(self.srcdir, src),
                                     path.join(self.outdir, dest))
@@ -473,7 +484,7 @@ def default_latex_engine(config: Config) -> str:
         return 'pdflatex'
 
 
-def default_latex_docclass(config: Config) -> Dict[str, str]:
+def default_latex_docclass(config: Config) -> dict[str, str]:
     """ Better default latex_docclass settings for specific languages. """
     if config.language == 'ja':
         if config.latex_engine == 'uplatex':
@@ -491,7 +502,7 @@ def default_latex_use_xindy(config: Config) -> bool:
     return config.latex_engine in {'xelatex', 'lualatex'}
 
 
-def default_latex_documents(config: Config) -> List[Tuple[str, str, str, str, str]]:
+def default_latex_documents(config: Config) -> list[tuple[str, str, str, str, str]]:
     """ Better default latex_documents settings. """
     project = texescape.escape(config.project, config.latex_engine)
     author = texescape.escape(config.author, config.latex_engine)
@@ -502,7 +513,7 @@ def default_latex_documents(config: Config) -> List[Tuple[str, str, str, str, st
              config.latex_theme)]
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.setup_extension('sphinx.builders.latex.transforms')
 
     app.add_builder(LaTeXBuilder)
@@ -510,25 +521,26 @@ def setup(app: Sphinx) -> Dict[str, Any]:
     app.connect('config-inited', validate_latex_theme_options, priority=800)
     app.connect('builder-inited', install_packages_for_ja)
 
-    app.add_config_value('latex_engine', default_latex_engine, None,
+    app.add_config_value('latex_engine', default_latex_engine, False,
                          ENUM('pdflatex', 'xelatex', 'lualatex', 'platex', 'uplatex'))
-    app.add_config_value('latex_documents', default_latex_documents, None)
-    app.add_config_value('latex_logo', None, None, [str])
-    app.add_config_value('latex_appendices', [], None)
-    app.add_config_value('latex_use_latex_multicolumn', False, None)
-    app.add_config_value('latex_use_xindy', default_latex_use_xindy, None, [bool])
-    app.add_config_value('latex_toplevel_sectioning', None, None,
+    app.add_config_value('latex_documents', default_latex_documents, False)
+    app.add_config_value('latex_logo', None, False, [str])
+    app.add_config_value('latex_appendices', [], False)
+    app.add_config_value('latex_use_latex_multicolumn', False, False)
+    app.add_config_value('latex_use_xindy', default_latex_use_xindy, False, [bool])
+    app.add_config_value('latex_toplevel_sectioning', None, False,
                          ENUM(None, 'part', 'chapter', 'section'))
-    app.add_config_value('latex_domain_indices', True, None, [list])
-    app.add_config_value('latex_show_urls', 'no', None)
-    app.add_config_value('latex_show_pagerefs', False, None)
-    app.add_config_value('latex_elements', {}, None)
-    app.add_config_value('latex_additional_files', [], None)
-    app.add_config_value('latex_theme', 'manual', None, [str])
-    app.add_config_value('latex_theme_options', {}, None)
-    app.add_config_value('latex_theme_path', [], None, [list])
+    app.add_config_value('latex_domain_indices', True, False, [list])
+    app.add_config_value('latex_show_urls', 'no', False)
+    app.add_config_value('latex_show_pagerefs', False, False)
+    app.add_config_value('latex_elements', {}, False)
+    app.add_config_value('latex_additional_files', [], False)
+    app.add_config_value('latex_table_style', ['booktabs', 'colorrows'], False, [list])
+    app.add_config_value('latex_theme', 'manual', False, [str])
+    app.add_config_value('latex_theme_options', {}, False)
+    app.add_config_value('latex_theme_path', [], False, [list])
 
-    app.add_config_value('latex_docclass', default_latex_docclass, None)
+    app.add_config_value('latex_docclass', default_latex_docclass, False)
 
     return {
         'version': 'builtin',
