@@ -1,17 +1,11 @@
-"""
-    sphinx.util.nodes
-    ~~~~~~~~~~~~~~~~~
+"""Docutils node-related utility functions for Sphinx."""
 
-    Docutils node-related utility functions for Sphinx.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
+import contextlib
 import re
 import unicodedata
-from typing import (TYPE_CHECKING, Any, Callable, Iterable, List, Optional, Set, Tuple, Type,
-                    Union, cast)
+from typing import TYPE_CHECKING, Any, Callable, Iterable
 
 from docutils import nodes
 from docutils.nodes import Element, Node
@@ -38,7 +32,7 @@ caption_ref_re = explicit_title_re  # b/w compat alias
 
 
 class NodeMatcher:
-    """A helper class for Node.traverse().
+    """A helper class for Node.findall().
 
     It checks that the given node is an instance of the specified node-classes and
     has the specified node-attributes.
@@ -47,19 +41,20 @@ class NodeMatcher:
     and ``reftype`` attributes::
 
         matcher = NodeMatcher(nodes.reference, refdomain='std', reftype='citation')
-        doctree.traverse(matcher)
+        doctree.findall(matcher)
         # => [<reference ...>, <reference ...>, ...]
 
     A special value ``typing.Any`` matches any kind of node-attributes.  For example,
     following example searches ``reference`` node having ``refdomain`` attributes::
 
-        from typing import Any
+        from __future__ import annotations
+from typing import Any
         matcher = NodeMatcher(nodes.reference, refdomain=Any)
-        doctree.traverse(matcher)
+        doctree.findall(matcher)
         # => [<reference ...>, <reference ...>, ...]
     """
 
-    def __init__(self, *node_classes: Type[Node], **attrs: Any) -> None:
+    def __init__(self, *node_classes: type[Node], **attrs: Any) -> None:
         self.classes = node_classes
         self.attrs = attrs
 
@@ -96,7 +91,7 @@ def get_full_module_name(node: Node) -> str:
     :param nodes.Node node: target node
     :return: full module dotted path
     """
-    return '{}.{}'.format(node.__module__, node.__class__.__name__)
+    return f'{node.__module__}.{node.__class__.__name__}'
 
 
 def repr_domxml(node: Node, length: int = 80) -> str:
@@ -147,13 +142,19 @@ def apply_source_workaround(node: Element) -> None:
         logger.debug('[i18n] PATCH: %r to have rawsource: %s',
                      get_full_module_name(node), repr_domxml(node))
         # strip classifier from rawsource of term
-        for classifier in reversed(list(node.parent.traverse(nodes.classifier))):
+        for classifier in reversed(list(node.parent.findall(nodes.classifier))):
             node.rawsource = re.sub(r'\s*:\s*%s' % re.escape(classifier.astext()),
                                     '', node.rawsource)
+    if isinstance(node, nodes.topic) and node.source is None:
+        # docutils-0.18 does not fill the source attribute of topic
+        logger.debug('[i18n] PATCH: %r to have source, line: %s',
+                     get_full_module_name(node), repr_domxml(node))
+        node.source, node.line = node.parent.source, node.parent.line
 
     # workaround: literal_block under bullet list (#4913)
     if isinstance(node, nodes.literal_block) and node.source is None:
-        node.source = get_node_source(node)
+        with contextlib.suppress(ValueError):
+            node.source = get_node_source(node)
 
     # workaround: recommonmark-0.2.0 doesn't set rawsource attribute
     if not node.rawsource:
@@ -185,14 +186,6 @@ IGNORED_NODES = (
 )
 
 
-def is_pending_meta(node: Node) -> bool:
-    if (isinstance(node, nodes.pending) and
-       isinstance(node.details.get('nodes', [None])[0], addnodes.meta)):
-        return True
-    else:
-        return False
-
-
 def is_translatable(node: Node) -> bool:
     if isinstance(node, addnodes.translatable):
         return True
@@ -228,9 +221,7 @@ def is_translatable(node: Node) -> bool:
             return False
         return True
 
-    if isinstance(node, addnodes.meta):
-        return True
-    if is_pending_meta(node):
+    if isinstance(node, nodes.meta):  # type: ignore
         return True
 
     return False
@@ -245,14 +236,11 @@ LITERAL_TYPE_NODES = (
 IMAGE_TYPE_NODES = (
     nodes.image,
 )
-META_TYPE_NODES = (
-    addnodes.meta,
-)
 
 
-def extract_messages(doctree: Element) -> Iterable[Tuple[Element, str]]:
+def extract_messages(doctree: Element) -> Iterable[tuple[Element, str]]:
     """Extract translatable messages from a document tree."""
-    for node in doctree.traverse(is_translatable):  # type: Element
+    for node in doctree.findall(is_translatable):  # type: Element
         if isinstance(node, addnodes.translatable):
             for msg in node.extract_original_messages():
                 yield node, msg
@@ -268,10 +256,8 @@ def extract_messages(doctree: Element) -> Iterable[Tuple[Element, str]]:
                 msg = '.. image:: %s' % node['uri']
             else:
                 msg = ''
-        elif isinstance(node, META_TYPE_NODES):
-            msg = node.rawcontent
-        elif isinstance(node, nodes.pending) and is_pending_meta(node):
-            msg = node.details['nodes'][0].rawcontent
+        elif isinstance(node, nodes.meta):  # type: ignore
+            msg = node["content"]
         else:
             msg = node.rawsource.replace('\n', ' ').strip()
 
@@ -280,18 +266,18 @@ def extract_messages(doctree: Element) -> Iterable[Tuple[Element, str]]:
             yield node, msg
 
 
-def get_node_source(node: Element) -> Optional[str]:
+def get_node_source(node: Element) -> str:
     for pnode in traverse_parent(node):
         if pnode.source:
             return pnode.source
-    return None
+    raise ValueError("node source not found")
 
 
-def get_node_line(node: Element) -> Optional[int]:
+def get_node_line(node: Element) -> int:
     for pnode in traverse_parent(node):
         if pnode.line:
             return pnode.line
-    return None
+    raise ValueError("node line not found")
 
 
 def traverse_parent(node: Element, cls: Any = None) -> Iterable[Element]:
@@ -301,7 +287,7 @@ def traverse_parent(node: Element, cls: Any = None) -> Iterable[Element]:
         node = node.parent
 
 
-def get_prev_node(node: Node) -> Optional[Node]:
+def get_prev_node(node: Node) -> Node | None:
     pos = node.parent.index(node)
     if pos > 0:
         return node.parent[pos - 1]
@@ -309,9 +295,12 @@ def get_prev_node(node: Node) -> Optional[Node]:
         return None
 
 
-def traverse_translatable_index(doctree: Element) -> Iterable[Tuple[Element, List["IndexEntry"]]]:  # NOQA
+def traverse_translatable_index(
+    doctree: Element
+) -> Iterable[tuple[Element, list[IndexEntry]]]:
     """Traverse translatable index node from a document tree."""
-    for node in doctree.traverse(NodeMatcher(addnodes.index, inline=False)):  # type: addnodes.index  # NOQA
+    matcher = NodeMatcher(addnodes.index, inline=False)
+    for node in doctree.findall(matcher):  # type: addnodes.index
         if 'raw_entries' in node:
             entries = node['raw_entries']
         else:
@@ -341,14 +330,14 @@ def nested_parse_with_titles(state: Any, content: StringList, node: Node) -> str
 def clean_astext(node: Element) -> str:
     """Like node.astext(), but ignore images."""
     node = node.deepcopy()
-    for img in node.traverse(nodes.image):
+    for img in node.findall(nodes.image):
         img['alt'] = ''
-    for raw in node.traverse(nodes.raw):
+    for raw in list(node.findall(nodes.raw)):
         raw.parent.remove(raw)
     return node.astext()
 
 
-def split_explicit_title(text: str) -> Tuple[bool, str, str]:
+def split_explicit_title(text: str) -> tuple[bool, str, str]:
     """Split role content into title and target, if given."""
     match = explicit_title_re.match(text)
     if match:
@@ -362,10 +351,10 @@ indextypes = [
 
 
 def process_index_entry(entry: str, targetid: str
-                        ) -> List[Tuple[str, str, str, str, Optional[str]]]:
+                        ) -> list[tuple[str, str, str, str, str | None]]:
     from sphinx.domains.python import pairindextypes
 
-    indexentries: List[Tuple[str, str, str, str, Optional[str]]] = []
+    indexentries: list[tuple[str, str, str, str, str | None]] = []
     entry = entry.strip()
     oentry = entry
     main = ''
@@ -400,15 +389,15 @@ def process_index_entry(entry: str, targetid: str
     return indexentries
 
 
-def inline_all_toctrees(builder: "Builder", docnameset: Set[str], docname: str,
-                        tree: nodes.document, colorfunc: Callable, traversed: List[str]
+def inline_all_toctrees(builder: Builder, docnameset: set[str], docname: str,
+                        tree: nodes.document, colorfunc: Callable, traversed: list[str]
                         ) -> nodes.document:
     """Inline all toctrees in the *tree*.
 
     Record all docnames in *docnameset*, and output docnames with *colorfunc*.
     """
-    tree = cast(nodes.document, tree.deepcopy())
-    for toctreenode in tree.traverse(addnodes.toctree):
+    tree = tree.deepcopy()
+    for toctreenode in list(tree.findall(addnodes.toctree)):
         newnodes = []
         includefiles = map(str, toctreenode['includefiles'])
         for includefile in includefiles:
@@ -426,7 +415,7 @@ def inline_all_toctrees(builder: "Builder", docnameset: Set[str], docname: str,
                 else:
                     sof = addnodes.start_of_file(docname=includefile)
                     sof.children = subtree.children
-                    for sectionnode in sof.traverse(nodes.section):
+                    for sectionnode in sof.findall(nodes.section):
                         if 'docname' not in sectionnode:
                             sectionnode['docname'] = includefile
                     newnodes.append(sof)
@@ -507,8 +496,8 @@ _non_id_translate_digraphs = {
 }
 
 
-def make_id(env: "BuildEnvironment", document: nodes.document,
-            prefix: str = '', term: str = None) -> str:
+def make_id(env: BuildEnvironment, document: nodes.document,
+            prefix: str = '', term: str | None = None) -> str:
     """Generate an appropriate node_id for given *prefix* and *term*."""
     node_id = None
     if prefix:
@@ -534,18 +523,18 @@ def make_id(env: "BuildEnvironment", document: nodes.document,
 
 
 def find_pending_xref_condition(node: addnodes.pending_xref, condition: str
-                                ) -> Optional[Element]:
+                                ) -> Element | None:
     """Pick matched pending_xref_condition node up from the pending_xref."""
     for subnode in node:
         if (isinstance(subnode, addnodes.pending_xref_condition) and
                 subnode.get('condition') == condition):
             return subnode
-    else:
-        return None
+    return None
 
 
-def make_refnode(builder: "Builder", fromdocname: str, todocname: str, targetid: str,
-                 child: Union[Node, List[Node]], title: str = None) -> nodes.reference:
+def make_refnode(builder: Builder, fromdocname: str, todocname: str, targetid: str | None,
+                 child: Node | list[Node], title: str | None = None
+                 ) -> nodes.reference:
     """Shortcut to create a reference node."""
     node = nodes.reference('', '', internal=True)
     if fromdocname == todocname and targetid:
@@ -601,9 +590,9 @@ def is_smartquotable(node: Node) -> bool:
     return True
 
 
-def process_only_nodes(document: Node, tags: "Tags") -> None:
+def process_only_nodes(document: Node, tags: Tags) -> None:
     """Filter ``only`` nodes which do not match *tags*."""
-    for node in document.traverse(addnodes.only):
+    for node in document.findall(addnodes.only):
         try:
             ret = tags.eval_condition(node['expr'])
         except Exception as err:
@@ -619,19 +608,3 @@ def process_only_nodes(document: Node, tags: "Tags") -> None:
                 # the only node, so we make sure docutils can transfer the id to
                 # something, even if it's just a comment and will lose the id anyway...
                 node.replace_self(nodes.comment())
-
-
-def _new_copy(self: Element) -> Element:
-    """monkey-patch Element.copy to copy the rawsource and line
-    for docutils-0.16 or older versions.
-
-    refs: https://sourceforge.net/p/docutils/patches/165/
-    """
-    newnode = self.__class__(self.rawsource, **self.attributes)
-    if isinstance(self, nodes.Element):
-        newnode.source = self.source
-        newnode.line = self.line
-    return newnode
-
-
-nodes.Element.copy = _new_copy  # type: ignore
