@@ -2,16 +2,19 @@
 
 import os
 import subprocess
-from subprocess import PIPE, CalledProcessError
+from pathlib import Path
+from subprocess import CalledProcessError
 from xml.etree import ElementTree
 
 import pytest
 
+from sphinx.builders.epub3 import _XML_NAME_PATTERN
+
 
 # check given command is runnable
 def runnable(command):
     try:
-        subprocess.run(command, stdout=PIPE, stderr=PIPE, check=True)
+        subprocess.run(command, capture_output=True, check=True)
         return True
     except (OSError, CalledProcessError):
         return False  # command not found or exit with non-zero
@@ -377,8 +380,33 @@ def test_run_epubcheck(app):
     if runnable(['java', '-version']) and os.path.exists(epubcheck):
         try:
             subprocess.run(['java', '-jar', epubcheck, app.outdir / 'SphinxTests.epub'],
-                           stdout=PIPE, stderr=PIPE, check=True)
+                           capture_output=True, check=True)
         except CalledProcessError as exc:
             print(exc.stdout.decode('utf-8'))
             print(exc.stderr.decode('utf-8'))
-            assert False, 'epubcheck exited with return code %s' % exc.returncode
+            raise AssertionError('epubcheck exited with return code %s' % exc.returncode)
+
+
+def test_xml_name_pattern_check():
+    assert _XML_NAME_PATTERN.match('id-pub')
+    assert _XML_NAME_PATTERN.match('webpage')
+    assert not _XML_NAME_PATTERN.match('1bfda21')
+
+
+@pytest.mark.sphinx('epub', testroot='images')
+def test_copy_images(app, status, warning):
+    app.build()
+
+    images_dir = Path(app.outdir) / '_images'
+    images = {image.name for image in images_dir.rglob('*')}
+    assert images == {
+        'img.gif',
+        'img.pdf',
+        'img.png',
+        'python-logo.png',
+        'rimg.png',
+        'rimg1.png',
+        'svgimg.pdf',
+        'svgimg.svg',
+        'testimäge.png',
+    }
