@@ -4,8 +4,9 @@ import os
 import re
 import subprocess
 from itertools import product
+from pathlib import Path
 from shutil import copyfile
-from subprocess import PIPE, CalledProcessError
+from subprocess import CalledProcessError
 
 import pytest
 
@@ -22,10 +23,11 @@ LATEX_ENGINES = ['pdflatex', 'lualatex', 'xelatex']
 DOCCLASSES = ['howto', 'manual']
 STYLEFILES = ['article.cls', 'fancyhdr.sty', 'titlesec.sty', 'amsmath.sty',
               'framed.sty', 'color.sty', 'fancyvrb.sty',
-              'fncychap.sty', 'geometry.sty', 'kvoptions.sty', 'hyperref.sty']
+              'fncychap.sty', 'geometry.sty', 'kvoptions.sty', 'hyperref.sty',
+              'booktabs.sty']
 
 LATEX_WARNINGS = ENV_WARNINGS + """\
-%(root)s/index.rst:\\d+: WARNING: unknown option: &option
+%(root)s/index.rst:\\d+: WARNING: unknown option: '&option'
 %(root)s/index.rst:\\d+: WARNING: citation not found: missing
 %(root)s/index.rst:\\d+: WARNING: a suitable image for latex builder not found: foo.\\*
 %(root)s/index.rst:\\d+: WARNING: Could not lex literal_block as "c". Highlighting skipped.
@@ -35,7 +37,7 @@ LATEX_WARNINGS = ENV_WARNINGS + """\
 # only run latex if all needed packages are there
 def kpsetest(*filenames):
     try:
-        subprocess.run(['kpsewhich'] + list(filenames), stdout=PIPE, stderr=PIPE, check=True)
+        subprocess.run(['kpsewhich'] + list(filenames), capture_output=True, check=True)
         return True
     except (OSError, CalledProcessError):
         return False  # command not found or exit with non-zero
@@ -54,14 +56,14 @@ def compile_latex_document(app, filename='python.tex'):
                     '--interaction=nonstopmode',
                     '-output-directory=%s' % app.config.latex_engine,
                     filename]
-            subprocess.run(args, stdout=PIPE, stderr=PIPE, check=True)
+            subprocess.run(args, capture_output=True, check=True)
     except OSError as exc:  # most likely the latex executable was not found
         raise pytest.skip.Exception from exc
     except CalledProcessError as exc:
-        print(exc.stdout)
-        print(exc.stderr)
-        assert False, '%s exited with return code %s' % (app.config.latex_engine,
-                                                         exc.returncode)
+        print(exc.stdout.decode('utf8'))
+        print(exc.stderr.decode('utf8'))
+        raise AssertionError(f'{app.config.latex_engine} exited with '
+                             f'return code {exc.returncode}')
 
 
 def skip_if_requested(testfunc):
@@ -90,6 +92,10 @@ def skip_if_stylefiles_notfound(testfunc):
 def test_build_latex_doc(app, status, warning, engine, docclass):
     app.config.latex_engine = engine
     app.config.latex_documents = [app.config.latex_documents[0][:4] + (docclass,)]
+    if engine == 'xelatex':
+        app.config.latex_table_style = ['booktabs']
+    elif engine == 'lualatex':
+        app.config.latex_table_style = ['colorrows']
     app.builder.init()
 
     LaTeXTranslator.ignore_missing_images = True
@@ -130,6 +136,17 @@ def test_writer(app, status, warning):
 
     assert 'Footnotes' not in result
 
+    assert ('\\begin{sphinxseealso}{See also}\n\n'
+            '\\sphinxAtStartPar\n'
+            'something, something else, something more\n'
+            '\\begin{description}\n'
+            '\\sphinxlineitem{\\sphinxhref{http://www.google.com}{Google}}\n'
+            '\\sphinxAtStartPar\n'
+            'For everything.\n'
+            '\n'
+            '\\end{description}\n'
+            '\n\n\\end{sphinxseealso}\n\n' in result)
+
 
 @pytest.mark.sphinx('latex', testroot='warnings', freshenv=True)
 def test_latex_warnings(app, status, warning):
@@ -413,7 +430,7 @@ def test_numref_with_prefix2(app, status, warning):
     'latex', testroot='numfig',
     confoverrides={'numfig': True, 'language': 'ja'})
 def test_numref_with_language_ja(app, status, warning):
-    app.builder.build_all()
+    app.build()
     result = (app.outdir / 'python.tex').read_text(encoding='utf8')
     print(result)
     print(status.getvalue())
@@ -723,7 +740,8 @@ def test_footnote(app, status, warning):
     assert '\\sphinxcite{footnote:bar}' in result
     assert ('\\bibitem[bar]{footnote:bar}\n\\sphinxAtStartPar\ncite\n') in result
     assert '\\sphinxcaption{Table caption \\sphinxfootnotemark[4]' in result
-    assert ('\\hline%\n\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
+    assert ('\\sphinxmidrule\n\\sphinxtableatstartofbodyhook%\n'
+            '\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
             'footnote in table caption\n%\n\\end{footnotetext}\\ignorespaces %\n'
             '\\begin{footnotetext}[5]\\sphinxAtStartFootnote\n'
             'footnote in table header\n%\n\\end{footnotetext}\\ignorespaces '
@@ -731,9 +749,9 @@ def test_footnote(app, status, warning):
             'VIDIOC\\_CROPCAP\n&\n\\sphinxAtStartPar\n') in result
     assert ('Information about VIDIOC\\_CROPCAP %\n'
             '\\begin{footnote}[6]\\sphinxAtStartFootnote\n'
-            'footnote in table not in header\n%\n\\end{footnote}\n\\\\\n\\hline\n'
-            '\\end{tabulary}\n'
-            '\\par\n\\sphinxattableend\\end{savenotes}\n') in result
+            'footnote in table not in header\n%\n\\end{footnote}\n\\\\\n'
+            '\\sphinxbottomrule\n\\end{tabulary}\n'
+            '\\sphinxtableafterendhook\\par\n\\sphinxattableend\\end{savenotes}\n') in result
 
 
 @pytest.mark.sphinx('latex', testroot='footnotes')
@@ -761,7 +779,8 @@ def test_reference_in_caption_and_codeblock_in_footnote(app, status, warning):
             'caption of normal table}\\label{\\detokenize{index:id36}}') in result
     assert ('\\caption{footnote \\sphinxfootnotemark[10] '
             'in caption \\sphinxfootnotemark[11] of longtable\\strut}') in result
-    assert ('\\endlastfoot\n%\n\\begin{footnotetext}[10]\\sphinxAtStartFootnote\n'
+    assert ('\\endlastfoot\n\\sphinxtableatstartofbodyhook\n%\n'
+            '\\begin{footnotetext}[10]\\sphinxAtStartFootnote\n'
             'Foot note in longtable\n%\n\\end{footnotetext}\\ignorespaces %\n'
             '\\begin{footnotetext}[11]\\sphinxAtStartFootnote\n'
             'Second footnote in caption of longtable\n') in result
@@ -992,7 +1011,7 @@ def test_image_in_section(app, status, warning):
 def test_latex_logo_if_not_found(app, status, warning):
     try:
         app.builder.build_all()
-        assert False  # SphinxError not raised
+        raise AssertionError()  # SphinxError not raised
     except Exception as exc:
         assert isinstance(exc, SphinxError)
 
@@ -1161,7 +1180,8 @@ def test_maxlistdepth_at_ten(app, status, warning):
     compile_latex_document(app, 'python.tex')
 
 
-@pytest.mark.sphinx('latex', testroot='latex-table')
+@pytest.mark.sphinx('latex', testroot='latex-table',
+                    confoverrides={'latex_table_style': []})
 @pytest.mark.test_params(shared_result='latex-table')
 def test_latex_table_tabulars(app, status, warning):
     app.builder.build_all()
@@ -1231,7 +1251,8 @@ def test_latex_table_tabulars(app, status, warning):
     assert actual == expected
 
 
-@pytest.mark.sphinx('latex', testroot='latex-table')
+@pytest.mark.sphinx('latex', testroot='latex-table',
+                    confoverrides={'latex_table_style': []})
 @pytest.mark.test_params(shared_result='latex-table')
 def test_latex_table_longtable(app, status, warning):
     app.builder.build_all()
@@ -1291,7 +1312,8 @@ def test_latex_table_longtable(app, status, warning):
     assert actual == expected
 
 
-@pytest.mark.sphinx('latex', testroot='latex-table')
+@pytest.mark.sphinx('latex', testroot='latex-table',
+                    confoverrides={'latex_table_style': []})
 @pytest.mark.test_params(shared_result='latex-table')
 def test_latex_table_complex_tables(app, status, warning):
     app.builder.build_all()
@@ -1309,12 +1331,34 @@ def test_latex_table_complex_tables(app, status, warning):
     expected = get_expected('gridtable')
     assert actual == expected
 
+    # grid table with tabularcolumns
+    # MEMO: filename should end with tabularcolumns but tabularcolumn has been
+    #       used in existing other cases
+    actual = tables['grid table with tabularcolumns having no vline']
+    expected = get_expected('gridtable_with_tabularcolumn')
+    assert actual == expected
+
     # complex spanning cell
     actual = tables['complex spanning cell']
     expected = get_expected('complex_spanning_cell')
     assert actual == expected
 
 
+@pytest.mark.sphinx('latex', testroot='latex-table')
+def test_latex_table_with_booktabs_and_colorrows(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'python.tex').read_text(encoding='utf8')
+    assert r'\PassOptionsToPackage{booktabs}{sphinx}' in result
+    assert r'\PassOptionsToPackage{colorrows}{sphinx}' in result
+    # tabularcolumns
+    assert r'\begin{longtable}[c]{|c|c|}' in result
+    # class: standard
+    assert r'\begin{tabulary}{\linewidth}[t]{|T|T|T|T|T|}' in result
+    assert r'\begin{longtable}[c]{ll}' in result
+    assert r'\begin{tabular}[t]{*{2}{\X{1}{2}}}' in result
+    assert r'\begin{tabular}[t]{\X{30}{100}\X{70}{100}}' in result
+
+
 @pytest.mark.sphinx('latex', testroot='latex-table',
                     confoverrides={'templates_path': ['_mytemplates/latex']})
 def test_latex_table_custom_template_caseA(app, status, warning):
@@ -1519,7 +1563,7 @@ def test_latex_labels(app, status, warning):
 def test_latex_figure_in_admonition(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'python.tex').read_text(encoding='utf8')
-    assert(r'\begin{figure}[H]' in result)
+    assert r'\begin{figure}[H]' in result
 
 
 def test_default_latex_documents():
@@ -1591,7 +1635,7 @@ def test_latex_elements_extrapackages(app, status, warning):
 @pytest.mark.sphinx('latex', testroot='nested-tables')
 def test_latex_nested_tables(app, status, warning):
     app.builder.build_all()
-    assert '' == warning.getvalue()
+    assert warning.getvalue() == ''
 
 
 @pytest.mark.sphinx('latex', testroot='latex-container')
@@ -1627,3 +1671,25 @@ def test_latex_code_role(app):
             common_content + '%\n}} code block') in content
     assert (r'\begin{sphinxVerbatim}[commandchars=\\\{\}]' +
             '\n' + common_content + '\n' + r'\end{sphinxVerbatim}') in content
+
+
+@pytest.mark.sphinx('latex', testroot='images')
+def test_copy_images(app, status, warning):
+    app.build()
+
+    test_dir = Path(app.outdir)
+    images = {
+        image.name for image in test_dir.rglob('*')
+        if image.suffix in {'.gif', '.pdf', '.png', '.svg'}
+    }
+    assert images == {
+        'img.gif',
+        'img.pdf',
+        'img.png',
+        'python-logo.png',
+        'rimg.png',
+        'rimg1.png',
+        'svgimg.pdf',
+        'svgimg.svg',
+        'testimäge.png',
+    }
