@@ -3,7 +3,8 @@
 import os
 import re
 import subprocess
-from subprocess import PIPE, CalledProcessError
+from pathlib import Path
+from subprocess import CalledProcessError
 from unittest.mock import Mock
 
 import pytest
@@ -17,7 +18,7 @@ from sphinx.writers.texinfo import TexinfoTranslator
 from .test_build_html import ENV_WARNINGS
 
 TEXINFO_WARNINGS = ENV_WARNINGS + """\
-%(root)s/index.rst:\\d+: WARNING: unknown option: &option
+%(root)s/index.rst:\\d+: WARNING: unknown option: '&option'
 %(root)s/index.rst:\\d+: WARNING: citation not found: missing
 %(root)s/index.rst:\\d+: WARNING: a suitable image for texinfo builder not found: foo.\\*
 %(root)s/index.rst:\\d+: WARNING: a suitable image for texinfo builder not found: \
@@ -49,13 +50,13 @@ def test_texinfo(app, status, warning):
     # now, try to run makeinfo over it
     try:
         args = ['makeinfo', '--no-split', 'sphinxtests.texi']
-        subprocess.run(args, stdout=PIPE, stderr=PIPE, cwd=app.outdir, check=True)
+        subprocess.run(args, capture_output=True, cwd=app.outdir, check=True)
     except OSError as exc:
         raise pytest.skip.Exception from exc  # most likely makeinfo was not found
     except CalledProcessError as exc:
         print(exc.stdout)
         print(exc.stderr)
-        assert False, 'makeinfo exited with return code %s' % exc.retcode
+        raise AssertionError('makeinfo exited with return code %s' % exc.retcode)
 
 
 @pytest.mark.sphinx('texinfo', testroot='markup-rubric')
@@ -137,3 +138,22 @@ def test_texinfo_samp_with_variable(app, status, warning):
     assert '@code{@var{variable_only}}' in output
     assert '@code{@var{variable} and text}' in output
     assert '@code{Show @var{variable} in the middle}' in output
+
+
+@pytest.mark.sphinx('texinfo', testroot='images')
+def test_copy_images(app, status, warning):
+    app.build()
+
+    images_dir = Path(app.outdir) / 'python-figures'
+    images = {image.name for image in images_dir.rglob('*')}
+    assert images == {
+        'img.gif',
+        'img.pdf',
+        'img.png',
+        'python-logo.png',
+        'rimg.png',
+        'rimg1.png',
+        'svgimg.pdf',
+        'svgimg.svg',
+        'testimäge.png',
+    }
