@@ -3,7 +3,6 @@
 import re
 import warnings
 
-import docutils
 import pytest
 from docutils import frontend, nodes, utils
 from docutils.parsers.rst import Parser as RstParser
@@ -17,7 +16,7 @@ from sphinx.testing.util import Struct, assert_node
 from sphinx.transforms import SphinxSmartQuotes
 from sphinx.util import texescape
 from sphinx.util.docutils import sphinx_domains
-from sphinx.writers.html import HTMLTranslator, HTMLWriter
+from sphinx.writers.html import HTML5Translator, HTMLWriter
 from sphinx.writers.latex import LaTeXTranslator, LaTeXWriter
 
 
@@ -82,7 +81,7 @@ class ForgivingTranslator:
         pass
 
 
-class ForgivingHTMLTranslator(HTMLTranslator, ForgivingTranslator):
+class ForgivingHTMLTranslator(HTML5Translator, ForgivingTranslator):
     pass
 
 
@@ -106,8 +105,7 @@ def verify_re_html(app, parse):
 def verify_re_latex(app, parse):
     def verify(rst, latex_expected):
         document = parse(rst)
-        app.builder = LaTeXBuilder(app)
-        app.builder.set_environment(app.env)
+        app.builder = LaTeXBuilder(app, app.env)
         app.builder.init()
         theme = app.builder.themes.get('manual')
         latex_translator = ForgivingLaTeXTranslator(document, app.builder, theme)
@@ -301,6 +299,14 @@ def get_verifier(verify, verify_re):
         ('\\sphinxAtStartPar\n'
          '\\sphinxkeyboard{\\sphinxupquote{Caps Lock}}'),
     ),
+    (
+        # kbd role
+        'verify',
+        ':kbd:`sys   rq`',
+        '<p><kbd class="kbd docutils literal notranslate">sys   rq</kbd></p>',
+        ('\\sphinxAtStartPar\n'
+         '\\sphinxkeyboard{\\sphinxupquote{sys   rq}}'),
+    ),
     (
         # non-interpolation of dashes in option role
         'verify_re',
@@ -359,27 +365,27 @@ def get_verifier(verify, verify_re):
         # description list: simple
         'verify',
         'term\n    description',
-        '<dl class="docutils">\n<dt>term</dt><dd>description</dd>\n</dl>',
+        '<dl class="simple">\n<dt>term</dt><dd><p>description</p>\n</dd>\n</dl>',
         None,
     ),
     (
         # description list: with classifiers
         'verify',
         'term : class1 : class2\n    description',
-        ('<dl class="docutils">\n<dt>term<span class="classifier">class1</span>'
-         '<span class="classifier">class2</span></dt><dd>description</dd>\n</dl>'),
+        ('<dl class="simple">\n<dt>term<span class="classifier">class1</span>'
+         '<span class="classifier">class2</span></dt><dd><p>description</p>\n</dd>\n</dl>'),
         None,
     ),
     (
         # glossary (description list): multiple terms
         'verify',
         '.. glossary::\n\n   term1\n   term2\n       description',
-        ('<dl class="glossary docutils">\n'
+        ('<dl class="simple glossary">\n'
          '<dt id="term-term1">term1<a class="headerlink" href="#term-term1"'
          ' title="Permalink to this term">¶</a></dt>'
          '<dt id="term-term2">term2<a class="headerlink" href="#term-term2"'
          ' title="Permalink to this term">¶</a></dt>'
-         '<dd>description</dd>\n</dl>'),
+         '<dd><p>description</p>\n</dd>\n</dl>'),
         None,
     ),
 ])
@@ -396,8 +402,6 @@ def test_inline(get_verifier, type, rst, html_expected, latex_expected):
         None,
     ),
 ])
-@pytest.mark.skipif(docutils.__version_info__ < (0, 16),
-                    reason='docutils-0.16 or above is required')
 def test_inline_docutils16(get_verifier, type, rst, html_expected, latex_expected):
     verifier = get_verifier(type)
     verifier(rst, html_expected, latex_expected)
