@@ -1,15 +1,8 @@
-"""
-    test_quickstart
-    ~~~~~~~~~~~~~~~
-
-    Test the sphinx.quickstart module.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Test the sphinx.quickstart module."""
 
 import time
 from io import StringIO
+from os import path
 
 import pytest
 
@@ -107,7 +100,7 @@ def test_quickstart_defaults(tempdir):
     conffile = tempdir / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    exec(conffile.read_text(), ns)
+    exec(conffile.read_text(encoding='utf8'), ns)  # NoQA: S102
     assert ns['extensions'] == []
     assert ns['templates_path'] == ['_templates']
     assert ns['project'] == 'Sphinx Test'
@@ -157,7 +150,7 @@ def test_quickstart_all_answers(tempdir):
     conffile = tempdir / 'source' / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    exec(conffile.read_text(), ns)
+    exec(conffile.read_text(encoding='utf8'), ns)  # NoQA: S102
     assert ns['extensions'] == [
         'sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx.ext.todo'
     ]
@@ -238,7 +231,7 @@ def test_default_filename(tempdir):
     conffile = tempdir / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    exec(conffile.read_text(), ns)
+    exec(conffile.read_text(encoding='utf8'), ns)  # NoQA: S102
 
 
 def test_extensions(tempdir):
@@ -248,5 +241,20 @@ def test_extensions(tempdir):
     conffile = tempdir / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    exec(conffile.read_text(), ns)
+    exec(conffile.read_text(encoding='utf8'), ns)  # NoQA: S102
     assert ns['extensions'] == ['foo', 'bar', 'baz']
+
+
+def test_exits_when_existing_confpy(monkeypatch):
+    # The code detects existing conf.py with path.isfile()
+    # so we mock it as True with pytest's monkeypatch
+    def mock_isfile(path):
+        return True
+    monkeypatch.setattr(path, 'isfile', mock_isfile)
+
+    qs.term_input = mock_input({
+        'Please enter a new root path (or just Enter to exit)': ''
+    })
+    d = {}
+    with pytest.raises(SystemExit):
+        qs.ask_user(d)
